<?php

namespace Drupal\mail_composer\Email;

use Drupal\Core\Extension\ModuleExtensionList;
use Twig\Error\LoaderError;
use Drupal\Core\StringTranslation\StringTranslationTrait;

/**
 * Class Email
 */
class Email implements EmailInterface {

  use StringTranslationTrait;

  /**
   * @var array
   */
  protected array $variables;

  /**
   * Email constructor.
   *
   * @param array $variables
   */
  public function __construct(array $variables = []) {
    $this->variables = $variables;
  }

  public function getFrom(): string {
    return (string) \Drupal::configFactory()->get('system.site')->get('mail');
  }

  public function getTo(): string {
    return '';
  }

  public function getReplyTo(): ?string {
    return NULL;
  }

  public function getSubject(): string {
    return '';
  }

  /**
   * Tries to load the email template corresponding to this class' name and the
   * current language.
   * If this email class is called SomeEmail, the template must be located
   * in the templates/emails folder inside the implementing module and be
   * called SomeEmail.twig.html (for the default language), or
   * SomeEmail-xx.twig.html for a specific language. If SomeEmail-xx.twig.html
   * is not found, SomeEmail.twig.html will be used. If no template is found,
   * an empty array will be returned.
   *
   * @return array
   *
   * @throws \Throwable
   * @throws \Twig\Error\RuntimeError
   * @throws \Twig\Error\SyntaxError
   */
  public function getBody(): array {
    $reflection_class = new \ReflectionClass($this);
    if (($namespace = $reflection_class->getName())
      && count($parts = explode('\\', $namespace)) > 1
      && $parts[0] === 'Drupal'
    ) {
      include_once \Drupal::root() . '/core/themes/engines/twig/twig.engine';

      /** @var ModuleExtensionList $module_extension_list */
      $module_extension_list = \Drupal::service('extension.list.module');

      /** @var \Twig\Environment $twig_service */
      $twig_service = \Drupal::service('twig');

      foreach (['-' . $this->getLangcode(), ''] as $langcode_suffix) {
        $template_path = $module_extension_list->getPath($parts[1])
          . '/templates/emails/' . $reflection_class->getShortName()
          . $langcode_suffix .'.html.twig';
        try {
          $twig_service->loadTemplate($template_path);
        }
        catch (LoaderError $ex) {
          continue;
        }

        // As this module uses twig templates in a non-standard way, this works
        // around 'Warning: Undefined array key "theme_hook_original" in
        // twig_render_template()'. An alternative would be to directly render
        // the templates, but then we loose the ability for our templates to be
        // altered by other modules, e.g. in case of HTML emails.
        if (!isset($this->variables['theme_hook_original'])) {
          $this->variables['theme_hook_original'] = '';
        }

        return [twig_render_template($template_path, $this->variables)];
      }
    }

    return [];
  }

  /**
   * Takes the current email template's class name and converts it to a Drupal
   * email key.
   *
   * Example: 'PetitionSubscriptionEmail' will be converted to
   * 'mail_composer_petition_subscription_email'.
   *
   * @return string
   */
  public function getKey(): string {
    return 'mail_composer_' . strtolower(preg_replace('/(?<!^)[A-Z]/', '_$0', (new \ReflectionClass($this))->getShortName()));
  }

  public function getLangcode(): string {
    return \Drupal::languageManager()->getCurrentLanguage()->getId();
  }

  public function getHeaders(): array {
    return [];
  }

}
